/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/


//
//  RWStream.h
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//

#pragma once
#include "FileStream.h"
#pragma warning(disable: 4293)

class CRWStream
{
private:
	unsigned char buffer[8];
    CFileStream* _stream;
public:
	CRWStream(void);
	~CRWStream(void);

    CRWStream(CFileStream* stream)
    {

        _stream = stream;
    }

    int Seek(long offset)
    {
        return _stream->Seek(offset, CFileStream::eBegin);
    }

    unsigned char ReadByte()
    {
        return (unsigned char)_stream->ReadByte();
    }

    unsigned short ReadUInt16()
    {
        _stream->Read(buffer, 0, 2);
        return (unsigned short)(buffer[0] | (buffer[1] << 8));
    }

    int ReadInt32()
    {
        _stream->Read(buffer, 0, 4);
        return (int)(buffer[0] | (buffer[1] << 8) | (buffer[2] << 16) | (buffer[3] << 24));
    }

    unsigned int ReadUInt32()
    {
        _stream->Read(buffer, 0, 4);
        return (unsigned int)(buffer[0] | (buffer[1] << 8) | (buffer[2] << 16) | (buffer[3] << 24));
    }

    DLONG ReadInt64()
    {
        _stream->Read(buffer, 0, 8);
        unsigned int ls = (unsigned int)(buffer[0] | (buffer[1] << 8) | (buffer[2] << 16) | (buffer[3] << 24));
        unsigned int ms = (unsigned int)((buffer[4]) | (buffer[5] << 8) | (buffer[6] << 16) | (buffer[7] << 24));
        return (DLONG)(((DLONG)ms << 32) | ls);
    }

    UDLONG ReadUInt64()
    {
        _stream->Read(buffer, 0, 8);
        return (UDLONG)(buffer[0] | ((UDLONG)buffer[1] << 8) | ((UDLONG)buffer[2] << 16) | ((UDLONG)buffer[3] << 24) | ((UDLONG)buffer[4] << 32) | ((UDLONG)buffer[5] << 40) | ((UDLONG)buffer[6] << 48) | ((UDLONG)buffer[7] << 56));
    }

    int ReadBytes(unsigned char* buffer, int count)
    {
        return _stream->Read(buffer, 0, count);
    }

    void Write(unsigned char b)
    {
        _stream->WriteByte(b);
    }

    void Write(unsigned short value)
    {
        buffer[0] = (unsigned char)value;
        buffer[1] = (unsigned char)(value >> 8);

        _stream->Write(buffer, 0, 2);
    }

    void Write(int value)
    {
        buffer[0] = (unsigned char)value;
        buffer[1] = (unsigned char)(value >> 8);
        buffer[2] = (unsigned char)(value >> 16);
        buffer[3] = (unsigned char)(value >> 24);

        _stream->Write(buffer, 0, 4);
    }

    void Write(DLONG value)
    {
        buffer[0] = (unsigned char)value;
        buffer[1] = (unsigned char)(value >> 8);
        buffer[2] = (unsigned char)(value >> 16);
        buffer[3] = (unsigned char)(value >> 24);
        buffer[4] = (unsigned char)(value >> 32);
        buffer[5] = (unsigned char)(value >> 40);
        buffer[6] = (unsigned char)(value >> 48);
        buffer[7] = (unsigned char)(value >> 56);

        _stream->Write(buffer, 0, 8);
    }

    void Write(unsigned int value)
    {
        buffer[0] = (unsigned char)value;
        buffer[1] = (unsigned char)(value >> 8);
        buffer[2] = (unsigned char)(value >> 16);
        buffer[3] = (unsigned char)(value >> 24);

        _stream->Write(buffer, 0, 4);
    }

    void Write(unsigned char* value, int length)
    {
        _stream->Write(value, 0, length);
    }

    void Close()
    {
        //Nothing to do :-)
    }
};

