/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
 */

//
//  FException.cpp
//  LIBCDF
//
//  Created by Weihua Luo on 4/3/2013.
//
#include "StdAfx.h"
#include "FException.h"
#include <stdlib.h>

CFException::CFException(void)
    : std::exception()
    , m_pMessage(nullptr)
{
    CopyMessage("CDF lib exception");
}

CFException::CFException(const char* message)
    : std::exception()
    , m_pMessage(nullptr)
{
    printf("%s\n", message);
    
    CopyMessage(message);
}

CFException::CFException(const std::string& message)
    : std::exception()
    , m_pMessage(nullptr)
{
    printf("%s\n", message.c_str());
    
    CopyMessage(message.c_str());
}

CFException::~CFException(void)
{
    if (m_pMessage != nullptr)
    {
        free(m_pMessage);
    }
}

const char * CFException::what() const throw()
{
	return m_pMessage;
}

void CFException::CopyMessage(const char *message)
{
    if (m_pMessage != nullptr)
    {
        free(m_pMessage);
    }
    
    //message shouldn't be null
    size_t len = ::strlen(message);
    m_pMessage = (char*)malloc(len + 1);
    if (m_pMessage != nullptr)
    {
        ::strcpy(m_pMessage, message);
    }
}
