/*
 "The contents of this file are subject to the Mozilla Public License
 
 Version 1.1 (the "License"); you may not use this file except in
 
 compliance with the License. You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 
 Software distributed under the License is distributed on an "AS IS"
 
 basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 
 License for the specific language governing rights and limitations
 
 under the License.
 
 
 The Original Code is OpenMCDF 1.5.4.
 
 
 The Initial Developer of the Original Code is Federico Blaseotto.
 
 Portions created by Federico Blaseotto are Copyright (C)2010 ~ 2013 Federico Blaseotto.
 
 All Rights Reserved.
 
 
 Contributor(s): Autodesk, Inc.."
*/


// LibCDF.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include "LibCDF.h"
#include <cstdlib>
#include <stdlib.h>	//ms_stdlib.h
#include <ctime>
#include "FileStream.h"

#ifndef RAND_MAX
#define RAND_MAX 0x7fff
#endif

//  caller is responsible for releasing the returned pointer
LIBCDF_EXPORT char* WCS2MBS(const wchar_t *pwcsName)
{
    size_t len = (::wcslen(pwcsName) + 1) * sizeof(wchar_t);
    char* pName = (char*)malloc(len);
	pName[0] = 0x0;
	size_t resCount = ::wcstombs(pName, pwcsName, len);
    return pName;
}

LIBCDF_EXPORT wchar_t* MBS2WCS(const char *pmbsName)
{
    size_t len = (::strlen(pmbsName) + 1) * sizeof(char);
    wchar_t* pName = (wchar_t*)malloc(len * sizeof(wchar_t));
	pName[0] = 0x0;
	pName[1] = 0x0;
	size_t resCount = ::mbstowcs(pName, pmbsName, len);
    return pName;
}

#ifdef __GNUC__

void UnixTimeToFileTime( time_t unix_time, S_FILETIME *filetime,unsigned long remainder )
{
	DLONG t = unix_time;
    t *= 10000000; // Convert from #seonds to #100-ns units
    t += 116444736000000000LL; // Convert from 1970 base to 1601 base (http://support.microsoft.com/kb/167296)
    t += remainder; // Add #milliseconds as unix_time doesn't account for that
    filetime->dwLowDateTime  = (unsigned int)t;
    filetime->dwHighDateTime = (unsigned int)(t >> 32);

}

LIBCDF_EXPORT bool SysTimeToFileTime(const S_SYSTEMTIME* lpSystemTime, LP_FILETIME lpFileTime)
{
    struct tm xtm;
    SysTime2Tm(lpSystemTime, xtm);
    time_t utctime = timegm(&xtm);
    UnixTimeToFileTime( utctime, lpFileTime, lpSystemTime->wMilliseconds * 10000 );
    return true;
}

#endif	//__GNUC__


std::string CFRandom::NextToString()
{
	int nRand = Next();
	char sRand[128] = {'\0'};
	sprintf(sRand, "%d", nRand);
	return std::string(sRand);	
}

std::wstring CFRandom::NextToWString(int maxValue)
{
	int nRand = Next(maxValue);
	wchar_t sRand[128] = {'\0'};
	swprintf(sRand, 127, L"%d", nRand);
	return std::wstring(sRand);	
}
int CFRandom::Next()
{
    std::srand((unsigned int)std::time(0)); //use current time as seed for random generator
    int random_variable = std::rand();
	return random_variable;
}

int CFRandom::Next(int maxValue)
{
	if (maxValue <= 0) maxValue = RAND_MAX;
    std::srand((unsigned int)std::time(0)); //use current time as seed for random generator
    int random_variable = std::rand();
	if (random_variable>maxValue) random_variable = random_variable%maxValue;
	return random_variable;
}

